import matplotlib.pyplot as plt
import numpy as np
import emcee
import corner
import pickle

## load data

alldata=np.loadtxt('gamma_TS10_4FGL_format.dat')
timeg=alldata[:,0]
fluxg=alldata[:,1]

alldata=np.loadtxt('OutburstR_residuals.dat',usecols=(0,1,2))
timeo=alldata[:,0]
fluxo=alldata[:,1]

alldata=np.loadtxt('BLLac_Pol_format.dat',usecols=(0,1,2,3,4,5,6))
time2=alldata[:,0]
pold=alldata[:,3]
polderr=alldata[:,4]
pola=alldata[:,5]
pola[pola>90]-=180
pola[pola<-90]+=180
polaerr=alldata[:,6]
timep=time2 -2400000.5


## only keep the part of data that is interesting

maskg=np.logical_and(timeg>=59076,timeg<=59089)
masko=np.logical_and(timeo>=59076,timeo<=59089)
maskp=np.logical_and(timep>=59076,timep<=59089)
timeemg, emg = timeg[maskg], fluxg[maskg]
timeemo, emo = timeo[masko], fluxo[masko]
timepol = timep[maskp]
pold, polderr = pold[maskp], polderr[maskp]
pola, polaerr = pola[maskp], polaerr[maskp]


## normalize everything to 0-1

normed_emg, normed_emo = emg/emg.max(), emo/emo.max()
normed_pd, normed_pderr = pold/pold.max(), polderr/pold.max()
normed_pa, normed_paerr = pola/pola.max(), polaerr/pola.max()
sigmaemg = sigmaemo = 0.1**2
sigmapd = normed_pderr**2
sigmapa = normed_paerr**2


## define variables that are fixed

doppler = 15.0**(3+1.2)
phase = 0.75*np.pi
bt = 1.; btsq = bt**2


## define the mcmc model
## assume uniform priors

def model_all(theta):
    ng, no, period, bp_amp, tb_amp, tbsq_avg = theta
    bp_emg = bp_amp*np.sin(timeemg/period*2.0*np.pi+phase)
    bp_emo = bp_amp*np.sin(timeemo/period*2.0*np.pi+phase)
    bp_pol = bp_amp*np.sin(timepol/period*2.0*np.pi+phase)
    
    tb_emg = tb_amp*np.sin(timeemg/period*2.0*np.pi+phase)
    tb_emo = tb_amp*np.sin(timeemo/period*2.0*np.pi+phase)
    tb_pol = tb_amp*np.sin(timepol/period*2.0*np.pi+phase)
    
    tb_off = -0.5*(tb_amp**2)
    em_avg = btsq + 0.5*(bp_amp**2) + tbsq_avg
    
    emg_temp = ng*doppler*(btsq + bp_emg**2 + tb_emg**2 + tbsq_avg + tb_off) / (em_avg*emg.max())
    emo_temp = no*doppler*(btsq + bp_emo**2 + tb_emo**2 + tbsq_avg + tb_off) / (em_avg*emo.max())
    pd_temp = (np.abs(btsq-bp_pol**2) / (btsq + bp_pol**2 + tb_pol**2 + tbsq_avg + tb_off) * 77) / pold.max()
    pa_temp = (np.arctan2(bp_pol, bt)/np.pi*180.0) / pola.max()
    
    return emg_temp, emo_temp, pd_temp, pa_temp


def lnprior_all(theta):
    ng, no, period, bp_amp, tb_amp, tbsq_avg = theta
    if period < 0. or bp_amp < 0. or ng < 0 or no < 0. or phase < 0.:
        return -np.inf
    if tb_amp < 0. or tbsq_avg < 0.:
        return -np.inf
    elif period > 1.4 or period < 0.8:
        return -np.inf
    return 0.

    
def lnlike_all(theta):
    ng, no, period, bp_amp, tb_amp, tbsq_avg = theta
    model = model_all(theta)
    lnlike_emg = -0.5 * np.sum((normed_emg - model[0])**2/sigmaemg + np.log(sigmaemg))
    lnlike_emo = -0.5 * np.sum((normed_emo - model[1])**2/sigmaemo + np.log(sigmaemo))
    lnlike_pd = -0.5 * np.sum((normed_pd - model[2])**2/sigmapd + np.log(sigmapd))
    lnlike_pa = -0.5 * np.sum((normed_pa - model[3])**2/sigmapa + np.log(sigmapa))
    return lnlike_emg+lnlike_emo+lnlike_pd+lnlike_pa


def lnprob_all(theta):
    lp = lnprior_all(theta)
    if not np.isfinite(lp):
        return -np.inf
    return lp + lnlike_all(theta)


## Initialize walkers

guess = np.array([0.0004, 0.0003, 1.2,  3, 4, 15])
nwalkers, ndim = 64, 6
pos = guess[np.newaxis] * (1. + 1e-2 * np.random.randn(nwalkers, ndim))
sampler = emcee.EnsembleSampler(
    nwalkers, ndim, lnprob_all, args=()
)
#run for some number of steps, we used 60000
sampler.run_mcmc(pos, 60000, progress=True)
#plot results as walkers
fig, axes = plt.subplots(ndim, figsize=(10, 2*ndim), sharex=True)
samples = sampler.get_chain()
labels = ['ng', 'no', 'period', 'bp_amp', 'tb_amp', 'tbsq_avg']
for i in range(ndim):
    ax = axes[i]
    ax.plot(samples[:, :, i], "k", alpha=0.3)
    ax.set_xlim(0, len(samples))
    ax.set_ylabel(labels[i])
    ax.yaxis.set_label_coords(-0.1, 0.5)
axes[-1].set_xlabel("step number")
fig.show()

#plot triangle plots, discard the first 30000 steps (burn-in)
flat_samples = sampler.get_chain(discard=30000, thin=10, flat=True)
quantiles = [0.16, 0.5, 0.84]
fig = corner.corner(
    flat_samples, labels=labels,
    quantiles=quantiles
);
plt.show()

#save the sampler if you want
#with open('mcmc_sampler.pkl', 'wb') as f:
#    pickle.dump(sampler, f)
#f.close()